(ns com.aurellem.run.save-corruption
  (:use (com.aurellem.gb gb-driver vbm))
  (:use (com.aurellem.run title)))

(use 'clojure.repl)

(defn-memo start-walking
  ([script] 
     (->> script
          (advance [:b] [:b :r])))
  ([] (start-walking (finish-title))))

(def move
  (partial advance []))

(defn-memo walk-to-stairs
  ([] (walk-to-stairs (start-walking)))
  ([script]
     (->> script
          (move [:u])
          (move [:u])
          (move [:u])
          (move [:u])
          (move [:u])
          (move [:r])
          (move [:r])
          (move [:r]))))

(defn-memo walk-to-door
  ([] (walk-to-door (walk-to-stairs)))
  ([script]
     (->> script
          (move [:d])
          (move [:d])
          (move [:d])
          (move [:d])
          (move [:d])
          (move [:d])
          (move [:l])
          (move [:l])
          (move [:l])
          (move [:l]))))

(defn-memo activate-menu
  ([] (activate-menu (walk-to-door)))
  ([script]
     (->> script
          (advance [:b] [:a :b :start]))))

(defn-memo save-game
  ([] (save-game (activate-menu)))
  ([script]
     (->> script
          (advance [] [:d])
          (play-moves [[] [] [] [:d] [] [] [] [:d] [] [] [:a]])
          scroll-text)))

(defn-memo corrupt-save
  ([] (corrupt-save (save-game)))
  ([script]
     (->>
      script
      (play-moves
       ;; this section is copied from speedrun-2942 and corrupts
       ;; the save so that the end-of-list marker for the pokemon
       ;; roster is destroyed, but the save is still playable.
       [[] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [:select] [:restart]]))))

(defn-memo start-game
  ([] (start-game (corrupt-save)))
  ([script]
     (->> script
          (title)
          (advance [] [:start])
          (advance [] [:a])
          (advance [:a] [:a :start]))))

(defn-memo destroy-item-end-of-list-marker
  ([] (destroy-item-end-of-list-marker (start-game)))
  ([script]
     (->>
      script
      (play-moves
       [[:start] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [:d] [] [] [] [:a] [] [] [:d] [] [] [:a] [] []
        [] [] [] [] [] [] [] [] [] [] [] [:d] [] [] [] []
        [:d] [] [] [] [] [:d] [] [] [] [] [:d] [] [] [] []
        [:d] [] [] [] [] [:d] [] [] [] [] [:d] [] [] [] []
        [:d] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:d]
        [] [] [] [:a] [] [] [:d] [] [] [:a] [] [] [] [] []
        [] [] [] [] [] [] [] [] [:u] [] [] [] [] [:u] [] []
        [] [:a] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] []]))))

(defn warp-to-elite-four
  "This is copied from speedrun-2942 to ensure that
   everything is good up to this point."
  []
  (->> (corrupt-save)
       (play-moves
        [ [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:a] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [:start] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [:start] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [:a] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [:start] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [:d] [] [] [] [:a] [] [] [:d] [] [] [:a] [] []
        [] [] [] [] [] [] [] [] [] [] [] [:d] [] [] [] []
        [:d] [] [] [] [] [:d] [] [] [] [] [:d] [] [] [] []
        [:d] [] [] [] [] [:d] [] [] [] [] [:d] [] [] [] []
        [:d] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:d]
        [] [] [] [:a] [] [] [:d] [] [] [:a] [] [] [] [] []
        [] [] [] [] [] [] [] [] [:u] [] [] [] [] [:u] [] []
        [] [:a] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [:b] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [:d] [] [] [:a] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [:d] [] [] [] [:d] [] [] []
        [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d]
        [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] []
        [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] []
        [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] []
        [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d]
        [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] []
        [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] []
        [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] []
        [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d]
        [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] []
        [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] []
        [] [] [] [] [] [] [] [] [] [:b] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:select]
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:b]
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [:d] [] [] [] [] [] [] [] [] [:b] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [:select] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [:b] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [:d] [] [] [] [] [] [] [] [] [] [] []
        [:b] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [:d] [] [] [] [] [:b] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [:select] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:b]
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:d]
        [] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [] [:d]
        [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] []
        [] [] [] [] [] [] [] [] [] [:select] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [:a] [] [] [:d] [] []
        [:a] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u]
        [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u]
        [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u]
        [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u]
        [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u]
        [] [:u] [] [:u] [:a] [] [] [] [] [] [:a] [] [] []
        [:a] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:b] [] []
        [] [] [] [] [] [] [:b] [:d] [:d] [:d] [:d] [:d] [:d]
        [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d]
        [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d]
        [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d]
        [:d] [:d] [:d] [:d] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] []
        [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:a] [] []
        [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] []
        [:a] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [:a] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] []
        [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:a]
        [] [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] []
        [] [:a] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [:b]])))
